/***************************************************************************
 *   Copyright (C) 2005 by Leon Pennington                                 *
 *   leon@leonscape.co.uk                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "gridstate.h"
#include <iostream>

GridState::GridState()
{
   setup();
}

GridState::GridState( const GridState &grid )
{
   setup();
   *this = grid;
}

GridState::~GridState()
{
   delete [] m_pCells;
   delete [] m_pBoxes;
   delete [] m_pCols;
   delete [] m_pRows;
}

const GridState& GridState::operator =( const GridState &grid )
{
   for( int i = 0; i < 81; ++i )
      m_pCells[i].value = grid.m_pCells[i].value;

   return *this;
}

bool GridState::operator ==( const GridState &grid ) const
{
   for( int i = 0; i < 81; ++i )
   {
      if( m_pCells[i].value != grid.m_pCells[i].value )
         return false;
   }

   return true;
}

bool GridState::operator !=( const GridState &grid ) const
{
   return !( *this == grid );
}

int GridState::filledCells() const
{
   int filled = 0;
   for( int i = 0; i < 81; ++i )
   {
      if( m_pCells[i].value >= 0 )
         ++filled;
   }

   return filled;
}

int GridState::simpleSolve()
{

   int i = 0;
   bool changed = true;
   for( ;changed; ++i )
   {
      changed = false;
      for( int i = 0; i < 81; ++i )
      {
         if( m_pCells[i].value < 0 )
         {
            std::list<int> poss = m_pCells[i].possible();
            if( poss.size() == 1 )
            {
               m_pCells[i].setValue( poss.front() );
               changed = true;
            }
         }
      }

      for( int i = 0; i < 9; ++i )
      {
         if( m_pBoxes[i].check() )
            changed = true;
         if( m_pCols[i].check() )
            changed = true;
         if( m_pRows[i].check() )
            changed = true;
      }
   }
   return i;
}

bool GridState::possible() const
{
   for( int i = 0; i < 81; ++i )
   {
      if( m_pCells[i].value < 0 )
      {
         std::list<int> poss = m_pCells[i].possible();
         if( poss.size() == 0 )
            return false;
      }
   }
   return true;
}

void GridState::setup()
{
   m_pCells = new Cell[81];
   m_pBoxes = new Part[9];
   m_pCols = new Part[9];
   m_pRows = new Part[9];

   int col, row, box;
   for( int i = 0; i < 81; ++i )
   {
      col = i % 9;
      m_pCells[i].col = &(m_pCols[ col ]);
      m_pCols[col].addCell( &m_pCells[i] );

      row = i / 9;
      m_pCells[i].row = &(m_pRows[ row ]);
      m_pRows[row].addCell( &m_pCells[i] );

      box = ( col / 3 ) + ( ( row / 3 ) * 3 );
      m_pCells[i].box = &(m_pBoxes[ box ]);
      m_pBoxes[box].addCell( &m_pCells[i] );
   }
}

GridState::Cell::Cell()
{
   value = -1;
   box = col = row = 0;
}

bool GridState::Cell::setValue( int val )
{
   if( val >= 0 && ( box->used( val ) || col->used( val ) || row->used( val ) ) )
       return false;

   value = val;
   return true;
}

std::list<int> GridState::Cell::possible() const
{
   std::list<int> poss;
   if( value < 0 )
   {
      for( int i = 0; i < 9; ++i )
      {
         if( !box->used( i ) && !col->used( i ) && !row->used( i ) )
            poss.push_back( i );
      }
   }

   return poss;
}

bool GridState::Cell::isPossible( int val ) const
{
   std::list<int> poss = possible();
   for( std::list<int>::iterator p = poss.begin(); p != poss.end(); ++p )
   {
      if( *p == val )
         return true;
   }

   return false;
}

GridState::Part::Part()
{
   for( int i = 0; i < 9; ++i )
      cells[i] = 0;
}

void GridState::Part::addCell( Cell *cell )
{
   for( int i = 0; i < 9; ++i )
   {
      if( !cells[i] )
      {
         cells[i] = cell;
         return;
      }
   }
}

bool GridState::Part::used( int val ) const
{
   for( int i = 0; i < 9; ++i )
   {
      if( cells[i]->value == val )
         return true;
   }

   return false;
}

bool GridState::Part::check()
{
   bool changed = false;
   for( int num = 0; num < 9; ++num )
   {
      if( !used( num ) )
      {
         Cell *fCell = 0;
         int poss = 0;
         for( int cell = 0; cell < 9; ++cell )
         {
            if( cells[cell]->isPossible( num ) )
            {
               fCell = cells[cell];
               ++poss;
            }

            if( poss > 1 )
               break;
         }

         if( poss == 1 )
         {
            fCell->value = num;
            changed = true;
         }
      }
   }
   return changed;
}
