/***************************************************************************
 *   Copyright (C) 2005 by Leon Pennington                                 *
 *   leon@leonscape.co.uk                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GRIDSTATE_H
#define GRIDSTATE_H

#include <list>

/**
 * This class holds all the information about a grid, and also helps to
 * solve the grid.
 */
class GridState
{
public:
   /// Constructor
   GridState();
   /// Copy Constructor
   GridState( const GridState &grid );
   /// Destructor
   ~GridState();

   /// Assignment operator
   const GridState& operator=( const GridState &grid );

   /// Comparison Operators
   bool operator==( const GridState &grid ) const;
   bool operator!=( const GridState &grid ) const;
   //bool operator<( const GridState &grid ) const;
   //bool operator>( const GridState &grid ) const;

   /// Returns the calue of a cell
   int value( int cell ) const { return m_pCells[cell].value; }

   /**
    * Sets a value of a cell
    * @return True if the value was valid otherwise false
    */
   bool setValue( int cell, int value ) const { return m_pCells[cell].setValue( value ); }

   /**
    * Sets a value of a cell in a box. used in generation
    */
   void setValue( int box, int cell, int value ) const { m_pBoxes[box].cells[cell]->value = value; }

   /// Returns a list of possible values for a cell
   std::list<int> possible( int cell ) const { return m_pCells[cell].possible(); }

   /**
    * Does a simple scanning solve on the grid
    * @return how many loops the solver went through
    */
   int simpleSolve();

   /// Checks to see if the grid is possible
   bool possible() const;

   /// Returns whether this grid is completley filled
   bool complete() const { return (filledCells() == 81 ); }

   /// Returns the number of cells that are filled
   int filledCells() const;

private:
   void setup();

   struct Part;
   struct Cell
   {
      int value;
      Part *box;
      Part *col;
      Part *row;

      /// Default Constructor
      Cell();

      /**
       * Sets the cells value
       * @return True if the cell could be set to this value
       */
      bool setValue( int val );

      /// Returns a list of possible values
      std::list<int> possible() const;

      /// Returns true if this value is possible in this cell
      bool isPossible( int val ) const;
   };


   struct Part
   {
      Cell *cells[9];

      /// Default Constructor
      Part();

      /// Adds a cell to the part
      void addCell( Cell *cell );

      /// Returns if this value has been used in this part
      bool used( int val ) const;

      /**
       * Checks the part for fillable cells
       * @return True if it found one or more cells til fill
       */
      bool check();
   };

   Cell *m_pCells;
   Part *m_pBoxes;
   Part *m_pCols;
   Part *m_pRows;
};

#endif
