/***************************************************************************
 *   Copyright (C) 2005 by Leon Pennington                                 *
 *   leon@leonscape.co.uk                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kudus.h"
#include "viewgrid.h"
#include "solver.h"
#include "generator.h"
#include "newdialog.h"

#include <qlayout.h>
#include <qpainter.h>

#include <kmainwindow.h>
#include <klocale.h>
#include <kstdgameaction.h>
#include <kaction.h>
#include <kedittoolbar.h>
#include <kstatusbar.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kprinter.h>

Kudus::Kudus() : KMainWindow( 0, "Kudus" )
{
   m_modified = false;
   setupWindow();
   setupActions();
   setStandardToolBarMenuEnabled( true );
   createStandardStatusBarAction();
   createGUI();
   setAutoSaveSettings();
   m_solveActive = false;
}

void Kudus::gameNew()
{
   NewDialog *dlg = new NewDialog( this );
   int code = dlg->exec();
   if( code == QDialog::Accepted )
   {
      Generator gen( dlg->difficulty() );
      m_startGrid = gen.generate();
      m_pGrid->setGrid( m_startGrid, ViewGrid::NEW );
      m_modified = false;
      Solver solve( m_startGrid );
      solve.findSolution();
      showTitle( solve.majorDifficulty(), solve.minorDifficulty(), i18n( "Generated" ) );
   }
}

void Kudus::gameRestart()
{
   if( m_modified )
   {
      m_pGrid->setGrid( m_startGrid, ViewGrid::NEW );
      m_modified = false;
   }

}

void Kudus::gameLoad()
{
   if( m_modified )
   {
      int ret = KMessageBox::warningYesNoCancel( 0, i18n( "Do you want to save your current game progress or discard it?" ),
            i18n( "Unsaved Game" ), i18n( "&Save" ), i18n( "&Discard" ) );
      switch( ret )
      {
         case KMessageBox::Yes:
            gameSave();
            break;
         case KMessageBox::Cancel:
            return;
            break;
      }
   }

   QString fileName = KFileDialog::getOpenFileName( QString::null, "*.kgc|" + i18n( "Kudus Grid" ) );
   if( !fileName.isEmpty() )
   {
      if( m_pGrid->loadGrid( fileName ) )
         m_modified = false;
   }
}

void Kudus::gameSave()
{
   QString fileName = KFileDialog::getSaveFileName( QString::null, "*.kgc|" + i18n( "Kudus Grid" ) );
   if( !fileName.isEmpty() )
   {
      if( !fileName.endsWith( ".kgc" ) )
         fileName += ".kgc";
      if ( QFile::exists( fileName ) )
      {
         if ( KMessageBox::warningContinueCancel( 0,
              i18n ("A file called \"%1\" already exists.\nDo you want to overwrite it?").arg( fileName ),
              QString::null, i18n ("Overwrite") ) == KMessageBox::Cancel )
            return;
      }

      if( m_pGrid->saveGrid( fileName ) )
         m_modified = false;
   }
}

void Kudus::gameClear()
{
   m_pGrid->clear();
   m_modified = false;
}


void Kudus::gamePrint()
{
   KPrinter printer;
   if ( printer.setup( this ) )
   {
      QPainter painter;
      painter.begin( &printer );
      m_pGrid->draw( painter );
      painter.end();
   }
}


void Kudus::moveSolve()
{
   Solver solve( m_pGrid->grid() );
   if( solve.findSolution() )
   {
      showTitle( solve.majorDifficulty(), solve.minorDifficulty(), i18n( "Solution" ) );
      m_solveActive = true;
      m_pGrid->setGrid( solve.solution(), ViewGrid::SOLUTION );
      m_solveActive = false;
   }
   else
      KMessageBox::sorry( this, i18n( "Could not find a solution." ) );
   m_modified = false;
}

void Kudus::movePencil()
{
   m_pGrid->showPossibles();
   m_modified = true;
}

void Kudus::moveHint()
{
   Solver solve( m_pGrid->grid() );
   if( solve.findSolution() )
      m_pGrid->setGrid( solve.solution(), ViewGrid::HINT );
   else
      KMessageBox::sorry( this, i18n( "Could not find a solution to get a hint from." ) );
   m_modified = true;
}

bool Kudus::queryClose()
{
   if( m_modified )
   {
      int ret = KMessageBox::warningYesNoCancel( 0, i18n( "Do you want to save your progress or discard it?" ),
            i18n( "Unsaved Changes" ), i18n( "&Save" ), i18n( "&Discard" ) );
      switch( ret )
      {
         case KMessageBox::Yes:
            gameSave();
            break;
         case KMessageBox::Cancel:
            return false;
            break;
      }
   }
   return true;
}

void Kudus::optionsConfigureToolbars()
{
   saveMainWindowSettings(KGlobal::config(), autoSaveGroup());
   KEditToolbar dlg(factory());
   connect(&dlg, SIGNAL(newToolbarConfig()), this, SLOT(applyNewToolbarConfig()));
   dlg.exec();
}

void Kudus::applyNewToolbarConfig()
{
   applyMainWindowSettings(KGlobal::config(), autoSaveGroup());
}

void Kudus::slotCompleted()
{
   if( !m_solveActive )
      KMessageBox::information( this, i18n( "Congratulations you've completed a puzzle!" ), i18n( "Congratulations" ) );
}

void Kudus::setupWindow()
{
   m_pGrid = new ViewGrid( this );
   setCentralWidget( m_pGrid );
   setMinimumSize( 300, 300 );
   connect( m_pGrid, SIGNAL(gridComplete()), SLOT(slotCompleted()) );
}

void Kudus::setupActions()
{
   // Game menu
   KStdGameAction::gameNew( this, SLOT(gameNew()), actionCollection() );
   KStdGameAction::load( this, SLOT(gameLoad()), actionCollection() );
   KStdGameAction::save( this, SLOT(gameSave()), actionCollection() );
   KStdGameAction::restart( this, SLOT(gameRestart()), actionCollection() );
   KStdGameAction::print( this, SLOT(gamePrint()), actionCollection() );
   KStdGameAction::quit( this, SLOT(close()), actionCollection() );
   new KAction( i18n( "Clear Grid" ), "fileclose", 0, this, SLOT( gameClear()), actionCollection(), "game_clear" );

   // move menu
   KStdGameAction::solve( this, SLOT(moveSolve()), actionCollection() );
   KStdGameAction::hint( this, SLOT(moveHint()), actionCollection() );
   new KAction( i18n( "Pencil" ), "pencil", 0, this, SLOT(movePencil()), actionCollection(), "move_pencil" );

   KStdAction::configureToolbars(this, SLOT(optionsConfigureToolbars()), actionCollection());
}

void Kudus::showTitle( int mjrDiff, int mnrDiff, const QString& type )
{
   setCaption( i18n( "%1 %2:%3" ).arg( type ).arg( mjrDiff ).arg( mnrDiff ) );
}

#include "kudus.moc"
