/***************************************************************************
 *   Copyright (C) 2005 by Leon Pennington                                 *
 *   leon@leonscape.co.uk                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "textcell.h"
#include "viewgrid.h"

#include <qstring.h>
#include <qdom.h>
#include <qpainter.h>

TextCell::TextCell( QWidget *parent, int position ) : KTextEdit(parent, 0 )
{
   m_grid = dynamic_cast<ViewGrid*>(parent);
   setMinimumSize( 30,30 );
   m_type = NORMAL;
   setPaletteBackgroundColor( Qt::white );
   setPaletteForegroundColor( Qt::black );
   m_position = position;
   m_col = position % 9;
   m_row = position / 9;
   m_box = ( m_col / 3 ) + ( ( m_row / 3 ) * 3 );
   setSizePolicy( QSizePolicy::Ignored, QSizePolicy::Ignored );
   setVScrollBarMode( QScrollView::AlwaysOff );
   setHScrollBarMode( QScrollView::AlwaysOff );
   setWrapPolicy( Anywhere );
   setTextFormat( Qt::PlainText );
   setAlignment( Qt::AlignHCenter );
   connect( this, SIGNAL(textChanged()), this, SLOT(slotTextChanged()) );
}

int TextCell::value() const
{
   if( text().length() == 1 )
      return ( text().toInt() - 1 );
   else
      return -1;
}

void TextCell::setValue( int val, CellType type )
{
   setType( type );
   if( val < 0 )
      setText( "" );
   else
      setText( QString::number( val + 1 ) );
}

void TextCell::setType( CellType type )
{
   switch( type )
   {
      case NONE:
         break;
      case NORMAL:
         m_type = NORMAL;
         setPaletteBackgroundColor( Qt::white );
         if( text().length() > 1 )
            setPaletteForegroundColor( Qt::gray );
         else
            setPaletteForegroundColor( Qt::black );
         break;
      case FIXED:
         m_type = FIXED;
         setPaletteBackgroundColor( Qt::gray );
         setPaletteForegroundColor( Qt::white );
         break;
      case GENERATED:
         m_type = GENERATED;
         setPaletteBackgroundColor( Qt::white );
         setPaletteForegroundColor( Qt::red );
         break;
   }
}

QDomElement TextCell::saveCell( QDomDocument &doc )
{
   QDomElement e = doc.createElement( "cell" );
   e.setAttribute( "type", m_type );
   e.setAttribute( "pos", m_position );
   QDomText t = doc.createTextNode( text() );
   e.appendChild( t );
   return e;
}

bool TextCell::loadCell( QDomElement &e )
{
   int type = e.attribute( "type", "0" ).toInt();
   setType( (CellType)type );
   setText( e.text() );
   return true;
}

void TextCell::draw( QPainter &p, const QRect &cell, const QRect &txtBox, const QFont &small, const QFont &large  )
{
   p.setPen( Qt::black );
   p.setBrush( paletteBackgroundColor() );
   p.drawRect( cell );
   p.flush();
   p.setPen( paletteForegroundColor() );
   if( text().length() > 1 )
   {
      p.setFont( small );
      p.drawText( txtBox, Qt::AlignTop|Qt::AlignLeft|Qt::BreakAnywhere, text() );
   }
   else
   {
      p.setFont( large );
      p.drawText( txtBox, Qt::AlignHCenter|Qt::AlignVCenter|Qt::SingleLine, text() );
   }
}

void TextCell::slotTextChanged()
{
   if( text().length() > 1 )
   {
      setPaletteForegroundColor( Qt::gray );
      setFont( m_grid->smallText() );
   }
   else
   {
      switch( m_type )
      {
         case NORMAL: setPaletteForegroundColor( Qt::black ); break;
         case FIXED: setPaletteForegroundColor( Qt::white ); break;
         case GENERATED: setPaletteForegroundColor( Qt::red ); break;
      }
      setFont( m_grid->largeText() );
   }
}

void TextCell::resizeEvent( QResizeEvent* )
{
   if( text().length() > 1 )
      setFont( m_grid->smallText() );
   else
      setFont( m_grid->largeText() );
}

void TextCell::keyPressEvent( QKeyEvent *e )
{
   bool accept = true;
   ViewGrid::FocusDirection dir;
   int csrIdx, csrPara;
   getCursorPosition( &csrPara, &csrIdx );
   switch( e->key() )
   {
      case Qt::Key_Up: dir = ViewGrid::UP; break;
      case Qt::Key_Down: dir = ViewGrid::DOWN; break;

      case Qt::Key_Left:
         if( csrIdx == 0 )
            dir = ViewGrid::LEFT;
         else
            accept = false;
         break;

      case Qt::Key_Right:
         if( csrIdx >= text().length() || m_type != NORMAL )
            dir = ViewGrid::RIGHT;
         else
            accept = false;
         break;

      case Qt::Key_Enter:
      case Qt::Key_Return:
      case Qt::Key_Tab: dir = ViewGrid::ONE; break;
      case Qt::Key_BackTab: dir = ViewGrid::BACK_ONE; break;

      case Qt::Key_Delete:
      case Qt::Key_BackSpace: accept = false; break;

      default:
         if( testText( e->text() ) )
            accept = false;
      break;
   }

   if( accept )
   {
      e->accept();
      m_grid->setCellFocus( m_position, dir );
   }
   else if( m_type == NORMAL )
      KTextEdit::keyPressEvent( e );
   else
      e->accept();
}

bool TextCell::testText( const QString &txt )
{
   bool ok;
   int val = txt.toInt( &ok );
   if( ok )
   {
      if( text().contains( txt ) > 0 )
      {
         if( hasSelectedText() && selectedText().contains( txt ) )
            return true;
         else
            return false;
      }
      else
         return true;
   }
   else
      return false;
}

#include "textcell.moc"
