/***************************************************************************
 *   Copyright (C) 2005 by Leon Pennington                                 *
 *   leon@leonscape.co.uk                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "viewgrid.h"
#include "textcell.h"
#include "gridstate.h"

#include <qfile.h>
#include <qdom.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <kglobalsettings.h>
#include <kdialog.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <cstdlib>

const float fileVersion = 0.1;

ViewGrid::ViewGrid(QWidget *parent, const char *name) : QWidget(parent, name)
{
   m_largeText = KGlobalSettings::generalFont();
   m_smallText = KGlobalSettings::generalFont();
   QGridLayout *mgl = new QGridLayout( this, 3, 3, KDialog::marginHint(), KDialog::spacingHint() );
   for( int j = 0; j < 3; ++j )
   {
      mgl->setColStretch( j, 1 );
      mgl->setRowStretch( j, 1 );
   }
   QGridLayout *grids[9];
   for( int i = 0; i < 9; ++i )
   {
      grids[i] = new QGridLayout( mgl, 3, 3, 0, 0 );
      for( int j = 0; j < 3; ++j )
      {
         grids[i]->setColStretch( j, 1 );
         grids[i]->setRowStretch( j, 1 );
      }
   }

   for( int i = 0; i < 81; ++i )
   {
      m_pCells[i] = new TextCell( this, i );
      grids[ m_pCells[i]->box() ]->addWidget( m_pCells[i], m_pCells[i]->row() % 3, m_pCells[i]->col() % 3 );
      connect( m_pCells[i], SIGNAL(textChanged()), SLOT(slotCellChanged()) );
   }
   setPaletteBackgroundColor( Qt::black );
}

GridState ViewGrid::grid()
{
   GridState grid;
   for( int i = 0; i < 81; ++i )
      grid.setValue( i, m_pCells[i]->value() );
   return grid;
}

void ViewGrid::setGrid( GridState grid, GridType type )
{
   switch( type )
   {
      case NEW:
         for( int i = 0; i < 81; ++i )
         {
            if( grid.value( i ) < 0 )
               m_pCells[i]->setValue( -1, TextCell::NORMAL );
            else
               m_pCells[i]->setValue( grid.value( i ), TextCell::FIXED );
         }
         break;
      case SOLUTION:
         for( int i = 0; i < 81; ++i )
         {
            if( m_pCells[i]->value() < 0 && grid.value( i ) >= 0 )
               m_pCells[i]->setValue( grid.value( i ), TextCell::GENERATED );
         }
         break;
      case HINT:
         {
            GridState cGrid = ViewGrid::grid();
            if( grid.filledCells() > cGrid.filledCells()  )
            {
               std::srand( time(0) );
               while( true )
               {
                  int rnd = ( (float)std::rand() / (float)RAND_MAX ) * 80.0;
                  if( m_pCells[rnd]->value() < 0 && grid.value( rnd ) >= 0 )
                  {
                     m_pCells[rnd]->setValue( grid.value( rnd ), TextCell::GENERATED );
                     break;
                  }
               }
            }
         }
         break;
   }
}

void ViewGrid::showPossibles()
{
   GridState grid = ViewGrid::grid();
   for( int i = 0; i < 81; ++i )
   {
      if( m_pCells[i]->value() < 0 )
      {
         QString str;
         std::list<int> poss = grid.possible( i );
         for( std::list<int>::iterator p = poss.begin(); p != poss.end(); ++p )
         {
            str += QString::number( *p + 1 );
         }
         m_pCells[i]->setText( str );
      }
   }
}

void ViewGrid::clear()
{
   for( int i = 0; i < 81; ++i )
      m_pCells[i]->setValue( -1, TextCell::NORMAL );
}

bool ViewGrid::saveGrid( const QString &fileName )
{
   QDomDocument doc( "Grid" );
   QDomElement root = doc.createElement( "Grid" );
   root.setAttribute( "version", fileVersion );
   doc.appendChild( root );
   for( int i = 0; i < 81; ++i )
   {
      QDomElement e = m_pCells[i]->saveCell( doc );
      root.appendChild( e );
   }

   QString xml = doc.toString( 4 );

   QFile file( fileName );
   if( file.open( IO_WriteOnly ) )
   {
      QTextStream stream( &file );
      stream << xml << endl;
      file.close();
      return true;
   }
   else
      KMessageBox::error( this, i18n( "Could not open the file %1 for writing." ).arg( fileName ) );
}

bool ViewGrid::loadGrid( const QString &fileName )
{
   QFile file( fileName );
   if( file.open( IO_ReadOnly ) )
   {
      QDomDocument doc( "Grid" );
      if( doc.setContent( &file ) )
      {
         QDomElement root = doc.documentElement();
         if ( root.attribute( "version", QString::number( fileVersion ) ).toFloat() == fileVersion )
         {
            QDomNode n = root.firstChild();
            while( !n.isNull() )
            {
               QDomElement e = n.toElement();
               if( !e.isNull() )
               {
                  int pos = e.attribute( "pos", "-1" ).toInt();
                  if( pos < 0 || !m_pCells[pos]->loadCell( e ) )
                  {
                     KMessageBox::sorry( this, i18n( "Sorry, There was an error reading this file. It maybe corrupted." ) );
                     break;
                  }
               }
               n = n.nextSibling();
            }
         }
         else
            KMessageBox::sorry( this, i18n( "Sorry, Cannot read files with a version higher than %1." ).arg( fileVersion ) );
      }
      else
         KMessageBox::error( this, i18n( "Could not parse the file %1." ).arg( fileName ) );
      file.close();
   }
   else
      KMessageBox::error( this, i18n( "Could not open the file %1 for reading." ).arg( fileName ) );

   return false;
}

void ViewGrid::setCellFocus( int currentCell, FocusDirection dir )
{
   int newCell = currentCell;
   bool done = false;
   while( !done )
   {
      switch( dir )
      {
         case UP: newCell -= 9; break;
         case DOWN: newCell += 9; break;
         case LEFT:
            if( m_pCells[newCell]->col() == 0 )
               newCell += 8;
            else
               newCell -= 1;
            break;
         case RIGHT:
            if( m_pCells[newCell]->col() == 8 )
               newCell -= 8;
            else
               newCell += 1;
            break;
         case ONE: newCell += 1; break;
         case BACK_ONE: newCell -=1; break;
      }

      if( newCell < 0 )
         newCell += 81;
      else if( newCell > 80 )
         newCell -= 81;

      if( m_pCells[newCell]->type() == TextCell::NORMAL )
      {
         m_pCells[newCell]->setFocus();
         done = true;
      }
      else if( newCell ==currentCell )
         done = true;
   }
}

void ViewGrid::draw( QPainter &p )
{
   /*
      grid calc is based on 290 Parts
      30 per cell 1 per cell border, 3 per box border and 4 per grid border
   */
   float sizeCalc = 290;

   QPaintDeviceMetrics m( p.device() );

   // Calculating out the offsets and box sizes
   int fullWidth = m.width() > m.height() ? m.height() : m.width();
   float gridSize = ( fullWidth / (int)sizeCalc ) * sizeCalc;
   float pageOffset = ( fullWidth - gridSize ) / 2;
   sizeCalc = gridSize / sizeCalc;
   float gridOffset = ( sizeCalc * 4 ) + pageOffset;
   float boxBorder = sizeCalc * 2;
   float cellSize = sizeCalc * 30;
   float cellOffSet = cellSize + sizeCalc;

   QFont printSmall, printLarge;
   printSmall = printLarge = m_largeText;

   // Calculating the font sizes
   float fontCalc = cellSize / 30.0;
   float fontOffset = ( fontCalc * 2.5 );
   float fontBoxSize = cellSize - ( fontOffset * 2.0 );
   printSmall.setPointSizeFloat( fontCalc * 3.0 );
   printLarge.setPointSizeFloat( fontCalc * 20.0 );

   p.setBrush( Qt::black );
   p.drawRect( pageOffset, pageOffset, gridSize, gridSize );
   p.setBrush( Qt::white );

   int col, row, box, boxX, boxY, offSetX, offSetY;
   QRect cell, fontBox;
   for( int i = 0; i < 81; ++i )
   {
      col = i % 9;
      row = i / 9;
      box = ( col / 3 ) + ( ( row / 3 ) * 3 );
      boxX = ( box % 3 ) * boxBorder;
      boxY = ( box / 3 ) * boxBorder;
      offSetX = gridOffset + ( cellOffSet * col ) + boxX;
      offSetY = gridOffset + ( cellOffSet * row ) + boxY;

      cell.setRect( offSetX, offSetY, cellSize, cellSize );
      fontBox.setRect( offSetX + fontOffset, offSetY + fontOffset, fontBoxSize, fontBoxSize );

      m_pCells[i]->draw( p, cell, fontBox, printSmall, printLarge );
   }
}

void ViewGrid::resizeEvent( QResizeEvent* )
{
   m_largeText.setPixelSize( height() / 14 );
   m_smallText.setPixelSize( m_largeText.pixelSize() / 3 );
}

void ViewGrid::slotCellChanged()
{
   GridState grid;
   for( int i = 0; i < 81; ++i )
   {
      if( m_pCells[i]->value() < 0 )
         return;
      else if( !grid.setValue( i, m_pCells[i]->value() ) )
         return;
   }

   emit gridComplete();
}

#include "viewgrid.moc"
